%
%This code uses crbm function defined in the code provided at "https://github.com/dustinstansbury/medal". They need to be included in the workspace. Natural Images from Imagenet dataset has been used for unsupervised training. 
%
%The labeled dataset is Caltech 101. Images from the 5 classes mentioned in the report are used for the SVM. For the SVM we used the libsvm library.


%---------------------------------------------------------------------------------
% LEVEL 1
%---------------------------------------------------------------------------------
dataset = 'ImageNet';
load trainData;
load imageData2;
%Load Imagenet Natural Images into training matrix
trainData = reshape(trainData',150,200,200);
dataSize = [150,200,1]; 
% DEFINE AN ARCHITECTURE for layer 1
arch = struct('dataSize', dataSize, ...
		'nFM', 20, ...
        'filterSize', [11 11], ...
        'stride', [1 1], ...
        'inputType', 'binary');
arch.opts = {'nEpoch', 1, ...
			 'lRate', .05, ...
			 'displayEvery',500, ...
			 'sparsity', .02, ...
			 'sparseGain', 5};%, ...
cr = crbm(arch);
cr = cr.train(trainData);
save useful cr,trainData;	% save useful data for later use
clear;
load useful;
ep = zeros(70,70,20,200);
for i=1:200,
	[cr,ep(:,:,:,i)] = cr.poolGivVis(trainData(:,:,i));
end
cr = cr.hidGivVis(trainData(:,:,100));	%random image chosen
% DISPLAY NETWORK FEATURES
figure;
[nRows,nCols,nFM]=size(cr.W);
W = reshape(cr.W,nRows*nCols,nFM);
subplot(141);
visWeights(W,1);
title('Learned Filters');
subplot(142);
imagesc(trainData(:,:,100)); colormap gray; axis image; axis off
title(sprintf('Sample Data\nPoint'));
subplot(143);
[nCols,nRows,k]=size(cr.eHid);
eh = reshape(cr.eHid,nRows*nCols,k);
visWeights(eh);
title('Feature Maps')
subplot(144);
[eY,eX,k] = size(ep(:,:,:,100));
visWeights(reshape(ep(:,:,:,100),eY*eX,k)); colormap gray
title(sprintf('Pooling Layer\nExpectations'))
drawnow

%---------------------------------------------------------------------------------
% LEVEL 2
%---------------------------------------------------------------------------------
save useful ep;
clear;
load useful;
trainData2 = ep;
clear ep;

dataSize2 = [70,70,20];  % [nY x nX x nChannels]

% DEFINE AN ARCHITECTURE
arch2 = struct('dataSize', dataSize2, ...
		'nFM', 100, ...
        'filterSize', [11 11], ...
        'stride', [1 1], ...
        'inputType', 'binary');
arch2.opts = {'nEpoch', 1, ...
			 'lRate', .05, ...
			 'displayEvery',500, ...
			 'sparsity', .02, ...
			 'sparseGain', 5};%, ...

% INITIALIZE AND TRAIN
cr2 = crbm(arch2);
cr2 = cr2.train(trainData2);
%Image feature show for 100th image
ep2 = cr2.poolGivVis(trainData2(:,:,100));
cr2 = cr2.hidGivVis(trainData2(:,:,100));
figure;
[nRows2,nCols2,nFM2]=size(cr2.W);
W2 = reshape(cr2.W,nRows2*nCols2,nFM2);

% visualization
%subplot(141);
%visWeights(W2,1);
%title('Learned Filters');
%
%subplot(142);
%imagesc(trainData(:,:,100)); colormap gray; axis image; axis off
%title(sprintf('Sample Data\nPoint'));
%
%subplot(143);
%[nCols2,nRows2,k2]=size(cr2.eHid);
%eh2 = reshape(cr2.eHid,nRows2*nCols2,k2);
%visWeights(eh2);
%title('Feature Maps')
%subplot(144);
%[eY2,eX2,k2] = size(ep2(:,:,100));
%visWeights(reshape(ep2(:,:,100),eY2*eX2,k2)); colormap gray
%title(sprintf('Pooling Layer\nExpectations'))
%drawnow

ep2 = zeros(30,42,100,200);

% extract featured using the representation learnt by the second layer and append to image data
augmented_data = zeros(200, 30*42*100 + 150*200);
for i=1:200,
	[cr,ep2(:,:,:,i)] = cr.poolGivVis(imageData(:,:,i));
        augmented_data(i,1:30000) = reshape(imageData(:,:,i),30000,1);
        augmented_data(i,30001:156000) = reshape(ep2(:,:,:,i),126000,1);
end


